==> feathers-mongoose-casl-dashboard-1.3.12.tgz: package/es/getDocFields.js <==
var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

import React from 'react';
import { docHelpers } from 'redux-admin'; //  ''redux-admin'';
import startCase from 'lodash/startCase';
import { getDeepObjectValue } from 'validate.js';
import { Collapse } from 'antd';
import RefComponent from './RefComponent';
import { getFieldName, getI18nLabelName, validateWhen } from './utils.js';
import { getCustomField } from './customFields';

/*
getDocField({
    key,
    type,
    label,
    required,
    ref,
    RefComponent = Reference,
    referenceKey,
    labelInValue,
    arrayItemType = String,
    multiSelect = false,
    disabled = false,
    documentRollConfig,
    isNewDoc,
    optionLabel,
    optionKey,
    options,
    getParamsByValue,
    placeholder,
    objectStructure
})
*/

var isAble = function isAble(_ref) {
  var isNewDoc = _ref.isNewDoc,
      item = _ref.item,
      itemKey = _ref.itemKey,
      preFix = _ref.preFix,
      _ref$abilityFields = _ref.abilityFields,
      abilityFields = _ref$abilityFields === undefined ? {} : _ref$abilityFields,
      ref = _ref.ref,
      dashboardData = _ref.dashboardData;
  var createFields = abilityFields.createFields,
      updateFields = abilityFields.updateFields;

  var fields = isNewDoc ? createFields : updateFields;
  var isAble = false;
  if (fields && fields.length) {
    var fieldsWithSelect = fields.filter(function (field) {
      return (typeof field === 'undefined' ? 'undefined' : _typeof(field)) === 'object' && field.path === itemKey;
    });
    if (fieldsWithSelect.length) {
      fieldsWithSelect.forEach(function (item) {
        if (preFix && preFix.length) {
          if (item.select) {
            isAble = item.select.includes(itemKey);
          } else {
            isAble = true; // For now we cant support field.when on the server
          }
        }
      });
    } else {
      var isDeep = preFix && preFix.length;
      isAble = fields.includes(itemKey) || fields.includes('*');
      if (!isAble && isDeep) {
        isAble = fields.includes(preFix + itemKey) || fields.includes('*');
      }
    }
    if (isAble && ref) {
      var isDashboardScreenAvailable = dashboardData.find(function (item) {
        return item.result.name === ref;
      });
      if (!isDashboardScreenAvailable) {
        console.log('src/screens/dashboard/getDocFields.js, remove fields from fields, user can"t see the ref');
        isAble = false;
      }
    }
    return isAble;
  }
  return true;
};
var getDocFields = function getDocFields(_ref2, jsonSchema, abilityFields, dashboardData) {
  var documentRollConfig = _ref2.documentRollConfig,
      isNewDoc = _ref2.isNewDoc,
      values = _ref2.values,
      lang = _ref2.lang,
      rtl = _ref2.rtl;
  var preFix = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '';
  var dashboardConfig = arguments[5];
  var customRenderField = arguments[6];
  var customElements = arguments[7];

  var properties = jsonSchema.properties || {};
  var requires = jsonSchema.required || [];
  var fields = [];
  var fieldsForLayout = [];
  var hasDocLayut = dashboardConfig && dashboardConfig.docLayout;
  Object.keys(properties).map(function (itemKey) {
    var item = properties[itemKey];
    var options = item.enum && item.enum.length ? item.enum : getDeepObjectValue(item, 'items.enum');
    var type = item.type;

    var objectStructure = void 0;
    if (item.format === 'date-time') type = Date;
    var objectFields = void 0;
    if (item.properties && Object.keys(item.properties).length) {
      objectFields = getDocFields({ documentRollConfig: documentRollConfig, isNewDoc: isNewDoc, values: values }, { properties: item.properties, required: requires }, abilityFields, dashboardData, itemKey + '.');
      if (objectFields.length) {
        var _currentField = React.createElement(
          Collapse,
          { key: itemKey, className: 'group-' + itemKey + ' ra-mb15', defaultActiveKey: isNewDoc ? ['1'] : null },
          React.createElement(
            Collapse.Panel,
            { key: '1', header: React.createElement(
                'label',
                null,
                startCase(itemKey),
                ':'
              ) },
            objectFields
          )
        );
        if (hasDocLayut) {
          fieldsForLayout[itemKey] = _currentField;
        } else {
          fields.push(_currentField);
        }
        return;
      }
    }
    var arrayItemType = getDeepObjectValue(item, 'items.type');
    var nestedArray = type === 'array' && !arrayItemType;
    if (type === 'array' && item.items && item.items.properties) {
      objectStructure = [];
      Object.keys(item.items.properties).forEach(function (itemKey) {
        var arrayField = item.items.properties[itemKey];
        var _type = arrayField.type;
        if (arrayField.format === 'date-time') {
          _type = 'date';
        }
        objectStructure.push({
          key: itemKey,
          label: getFieldName({ target: 'doc', lang: lang, itemKey: itemKey, dashboardDoc: dashboardDoc, dashboard: dashboard, dashboardConfig: dashboardConfig }),
          type: _type
        });
      });
    }
    var dashboard = getDeepObjectValue(item, 'meta.0.dashboard') || {};
    var dashboardDoc = getDeepObjectValue(item, 'meta.0.dashboard.doc') || {};
    if (dashboard.hide) return null;
    if (dashboardDoc.hide) return null;
    if (isNewDoc && dashboardDoc.hideOnCreate) return null;
    if (!isNewDoc && dashboardDoc.hideOnUpdate) return null;
    if (dashboardDoc.options) {
      // When we want to display enums with labels
      options = dashboardDoc.options;
      dashboardDoc.optionKey = dashboardDoc.optionKey || 'value';
      dashboardDoc.displayKey = dashboardDoc.displayKey || 'label';
    }
    var ref = getDeepObjectValue(item, 'meta.0.ref') || getDeepObjectValue(item, 'items.meta.0.ref');
    var showField = isAble({ isNewDoc: isNewDoc, item: item, itemKey: itemKey, preFix: preFix, abilityFields: abilityFields, ref: ref, dashboardData: dashboardData });
    if (!showField) return null;

    var fieldConfig = {
      key: preFix + itemKey,
      label: getFieldName({ target: 'doc', lang: lang, itemKey: itemKey, dashboardDoc: dashboardDoc, dashboard: dashboard, dashboardConfig: dashboardConfig }),
      type: type,
      required: requires.includes(preFix + itemKey),
      documentRollConfig: documentRollConfig,
      isNewDoc: isNewDoc,
      arrayItemType: arrayItemType,
      ref: ref,
      optionLabel: dashboardDoc.displayKey || getDeepObjectValue(item, 'meta.0.displayKey') || getDeepObjectValue(item, 'items.meta.0.displayKey'),
      optionKey: dashboardDoc.optionKey || '_id',
      multiSelect: item.type === 'array' || item.type === Array,
      options: dashboardDoc.options || options && options.length ? options : null,
      nestedArray: nestedArray,
      disabled: item.readOnly || dashboard.readOnly || dashboardDoc.readOnly || !isNewDoc && dashboardDoc.immutable,
      inputType: dashboardDoc.inputType,
      inputProps: dashboardDoc.inputProps ? JSON.parse(dashboardDoc.inputProps) : null,
      RefComponent: RefComponent,
      objectStructure: objectStructure,
      helpText: dashboardDoc.helpText
    };
    if (dashboardDoc.options) {
      fieldConfig.renderLabel = function (_ref3) {
        var label = _ref3.label,
            option = _ref3.option,
            optionKey = _ref3.optionKey;

        var optionFromOptions = dashboardDoc.options.find(function (o) {
          return option[optionKey] === o.value;
        });
        if (optionFromOptions && optionFromOptions.i18nLabels) {
          return optionFromOptions.i18nLabels[lang] || optionFromOptions.label || label;
        }
        return label;
      };
    }

    var currentField = docHelpers.getDocField(fieldConfig);
    if (hasDocLayut) {
      fieldsForLayout[itemKey] = currentField;
    } else {
      fields.push(currentField);
    }
  });
  if (dashboardConfig && dashboardConfig.docLayout) {

    var addItemField = function addItemField(item, key, _fields) {
      if (!item) return;
      var isArrayOfFields = Array.isArray(item);

      var getField = function getField(itemField, index) {
        var _key = key + (index || '');
        if ((typeof itemField === 'undefined' ? 'undefined' : _typeof(itemField)) === 'object') {
          if (isNewDoc && itemField.hideOnCreate) return null;
          if (!isNewDoc && itemField.hideOnUpdate) return null;
          if (itemField.when) {
            var fieldsEquale = [];
            var fieldsNotEquale = [];
            addItemField(itemField.when.then, key, fieldsEquale);
            addItemField(itemField.when.otherwise, key, fieldsNotEquale);
            return React.createElement(
              'span',
              { key: 'group-' + _key + '-withCondiation' },
              validateWhen(values, itemField) ? React.createElement(
                'span',
                { key: 'group-true=' + _key, className: 'ra-doc-layout-itemsGroup' },
                fieldsEquale
              ) : React.createElement(
                'span',
                { key: 'group-false=' + _key, className: 'ra-doc-layout-itemsGroup' },
                fieldsNotEquale
              )
            );
          }
          if (itemField.type === 'custom') {
            var displayLabel = getI18nLabelName({ lang: lang, itemKey: itemField.itemKey, dashboardConfig: dashboardConfig }) || itemField.label || itemField.itemKey;
            return React.createElement(
              'span',
              { key: _key },
              getCustomField({ item: itemField, key: itemField.itemKey, displayLabel: displayLabel, lang: lang, rtl: rtl, customRenderField: customRenderField, customElements: customElements })
            );
          }
        }
        return fieldsForLayout[itemField];
      };

      if (isArrayOfFields) {
        var fieldsToAdd = item.map(getField).filter(Boolean);
        _fields.push(React.createElement(
          'span',
          { className: 'ra-doc-layout-itemsGroup', key: 'group' + key },
          fieldsToAdd
        ));
      } else {
        var fieldToAdd = getField(item);
        if (fieldToAdd) {
          _fields.push(fieldToAdd);
        }
      }
    };
    dashboardConfig.docLayout.map(function (item, index) {
      return addItemField(item, index, fields);
    });
  }
  return fields;
};
export default getDocFields;

==> feathers-mongoose-casl-dashboard-1.3.15.tgz: package/es/getDocFields.js <==
var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

import React from 'react';
import { docHelpers } from 'redux-admin'; //  ''redux-admin'';
import startCase from 'lodash/startCase';
import { getDeepObjectValue } from 'validate.js';
import { Collapse } from 'antd';
import RefComponent from './RefComponent';
import { getFieldName, getI18nLabelName, validateWhen } from './utils.js';
import { getCustomField } from './customFields';

/*
getDocField({
    key,
    type,
    label,
    required,
    ref,
    RefComponent = Reference,
    referenceKey,
    labelInValue,
    arrayItemType = String,
    multiSelect = false,
    disabled = false,
    documentRollConfig,
    isNewDoc,
    optionLabel,
    optionKey,
    options,
    getParamsByValue,
    placeholder,
    objectStructure
})
*/

var isAble = function isAble(_ref) {
  var isNewDoc = _ref.isNewDoc,
      item = _ref.item,
      itemKey = _ref.itemKey,
      preFix = _ref.preFix,
      _ref$abilityFields = _ref.abilityFields,
      abilityFields = _ref$abilityFields === undefined ? {} : _ref$abilityFields,
      ref = _ref.ref,
      dashboardData = _ref.dashboardData;
  var createFields = abilityFields.createFields,
      updateFields = abilityFields.updateFields;

  var fields = isNewDoc ? createFields : updateFields;
  var isAble = false;
  if (fields && fields.length) {
    var fieldsWithSelect = fields.filter(function (field) {
      return (typeof field === 'undefined' ? 'undefined' : _typeof(field)) === 'object' && field.path === itemKey;
    });
    if (fieldsWithSelect.length) {
      fieldsWithSelect.forEach(function (item) {
        if (preFix && preFix.length) {
          if (item.select) {
            isAble = item.select.includes(itemKey);
          } else {
            isAble = true; // For now we cant support field.when on the server
          }
        }
      });
    } else {
      var isDeep = preFix && preFix.length;
      isAble = fields.includes(itemKey) || fields.includes('*');
      if (!isAble && isDeep) {
        isAble = fields.includes(preFix + itemKey) || fields.includes('*');
      }
    }
    if (isAble && ref) {
      var isDashboardScreenAvailable = dashboardData.find(function (item) {
        return item.result.name === ref;
      });
      if (!isDashboardScreenAvailable) {
        console.log('src/screens/dashboard/getDocFields.js, remove fields from fields, user can"t see the ref');
        isAble = false;
      }
    }
    return isAble;
  }
  return true;
};
var getDocFields = function getDocFields(_ref2, jsonSchema, abilityFields, dashboardData) {
  var documentRollConfig = _ref2.documentRollConfig,
      isNewDoc = _ref2.isNewDoc,
      values = _ref2.values,
      lang = _ref2.lang,
      rtl = _ref2.rtl;
  var preFix = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '';
  var dashboardConfig = arguments[5];
  var customRenderField = arguments[6];
  var customElements = arguments[7];

  var properties = jsonSchema.properties || {};
  var requires = jsonSchema.required || [];
  var fields = [];
  var fieldsForLayout = [];
  var hasDocLayut = dashboardConfig && dashboardConfig.docLayout;
  Object.keys(properties).map(function (itemKey) {
    var item = properties[itemKey];
    var options = item.enum && item.enum.length ? item.enum : getDeepObjectValue(item, 'items.enum');
    var type = item.type;

    var objectStructure = void 0;
    if (item.format === 'date-time') type = Date;
    var objectFields = void 0;
    if (item.properties && Object.keys(item.properties).length) {
      objectFields = getDocFields({ documentRollConfig: documentRollConfig, isNewDoc: isNewDoc, values: values }, { properties: item.properties, required: requires }, abilityFields, dashboardData, itemKey + '.');
      if (objectFields.length) {
        var _currentField = React.createElement(
          Collapse,
          { key: itemKey, className: 'group-' + itemKey + ' ra-mb15', defaultActiveKey: isNewDoc ? ['1'] : null },
          React.createElement(
            Collapse.Panel,
            { key: '1', header: React.createElement(
                'label',
                null,
                startCase(itemKey),
                ':'
              ) },
            objectFields
          )
        );
        if (hasDocLayut) {
          fieldsForLayout[itemKey] = _currentField;
        } else {
          fields.push(_currentField);
        }
        return;
      }
    }
    var arrayItemType = getDeepObjectValue(item, 'items.type');
    var nestedArray = type === 'array' && !arrayItemType;
    if (type === 'array' && item.items && item.items.properties) {
      objectStructure = [];
      Object.keys(item.items.properties).forEach(function (itemKey) {
        var arrayField = item.items.properties[itemKey];
        var _type = arrayField.type;
        if (arrayField.format === 'date-time') {
          _type = 'date';
        }
        objectStructure.push({
          key: itemKey,
          label: getFieldName({ target: 'doc', lang: lang, itemKey: itemKey, dashboardDoc: dashboardDoc, dashboard: dashboard, dashboardConfig: dashboardConfig }),
          type: _type
        });
      });
    }
    var dashboard = getDeepObjectValue(item, 'meta.0.dashboard') || {};
    var dashboardDoc = getDeepObjectValue(item, 'meta.0.dashboard.doc') || {};
    if (dashboard.hide) return null;
    if (dashboardDoc.hide) return null;
    if (isNewDoc && dashboardDoc.hideOnCreate) return null;
    if (!isNewDoc && dashboardDoc.hideOnUpdate) return null;
    if (dashboardDoc.options) {
      // When we want to display enums with labels
      options = dashboardDoc.options;
      dashboardDoc.optionKey = dashboardDoc.optionKey || 'value';
      dashboardDoc.displayKey = dashboardDoc.displayKey || 'label';
    }
    var ref = getDeepObjectValue(item, 'meta.0.ref') || getDeepObjectValue(item, 'items.meta.0.ref');
    var showField = isAble({ isNewDoc: isNewDoc, item: item, itemKey: itemKey, preFix: preFix, abilityFields: abilityFields, ref: ref, dashboardData: dashboardData });
    if (!showField) return null;

    var fieldConfig = {
      key: preFix + itemKey,
      label: getFieldName({ target: 'doc', lang: lang, itemKey: itemKey, dashboardDoc: dashboardDoc, dashboard: dashboard, dashboardConfig: dashboardConfig }),
      type: type,
      required: requires.includes(preFix + itemKey),
      documentRollConfig: documentRollConfig,
      isNewDoc: isNewDoc,
      arrayItemType: arrayItemType,
      ref: ref,
      optionLabel: dashboardDoc.displayKey || getDeepObjectValue(item, 'meta.0.displayKey') || getDeepObjectValue(item, 'items.meta.0.displayKey'),
      optionKey: dashboardDoc.optionKey || '_id',
      multiSelect: item.type === 'array' || item.type === Array,
      options: dashboardDoc.options || options && options.length ? options : null,
      nestedArray: nestedArray,
      disabled: item.readOnly || dashboard.readOnly || dashboardDoc.readOnly || !isNewDoc && dashboardDoc.immutable,
      inputType: dashboardDoc.inputType,
      inputProps: dashboardDoc.inputProps ? JSON.parse(dashboardDoc.inputProps) : null,
      RefComponent: RefComponent,
      objectStructure: objectStructure,
      helpText: dashboardDoc.helpText,
      customElements: customElements
    };
    if (dashboardDoc.options) {
      fieldConfig.renderLabel = function (_ref3) {
        var label = _ref3.label,
            option = _ref3.option,
            optionKey = _ref3.optionKey;

        var optionFromOptions = dashboardDoc.options.find(function (o) {
          return option[optionKey] === o.value;
        });
        if (optionFromOptions && optionFromOptions.i18nLabels) {
          return optionFromOptions.i18nLabels[lang] || optionFromOptions.label || label;
        }
        return label;
      };
    }

    var currentField = docHelpers.getDocField(fieldConfig);
    if (hasDocLayut) {
      fieldsForLayout[itemKey] = currentField;
    } else {
      fields.push(currentField);
    }
  });
  if (dashboardConfig && dashboardConfig.docLayout) {

    var addItemField = function addItemField(item, key, _fields) {
      if (!item) return;
      var isArrayOfFields = Array.isArray(item);

      var getField = function getField(itemField, index) {
        var _key = key + (index || '');
        if ((typeof itemField === 'undefined' ? 'undefined' : _typeof(itemField)) === 'object') {
          if (isNewDoc && itemField.hideOnCreate) return null;
          if (!isNewDoc && itemField.hideOnUpdate) return null;
          if (itemField.when) {
            var fieldsEquale = [];
            var fieldsNotEquale = [];
            addItemField(itemField.when.then, key, fieldsEquale);
            addItemField(itemField.when.otherwise, key, fieldsNotEquale);
            return React.createElement(
              'span',
              { key: 'group-' + _key + '-withCondiation' },
              validateWhen(values, itemField) ? React.createElement(
                'span',
                { key: 'group-true=' + _key, className: 'ra-doc-layout-itemsGroup' },
                fieldsEquale
              ) : React.createElement(
                'span',
                { key: 'group-false=' + _key, className: 'ra-doc-layout-itemsGroup' },
                fieldsNotEquale
              )
            );
          }
          if (itemField.type === 'custom') {
            var displayLabel = getI18nLabelName({ lang: lang, itemKey: itemField.itemKey, dashboardConfig: dashboardConfig }) || itemField.label || itemField.itemKey;
            return React.createElement(
              'span',
              { key: _key },
              getCustomField({ item: itemField, key: itemField.itemKey, displayLabel: displayLabel, lang: lang, rtl: rtl, customRenderField: customRenderField, customElements: customElements })
            );
          }
        }
        return fieldsForLayout[itemField];
      };

      if (isArrayOfFields) {
        var fieldsToAdd = item.map(getField).filter(Boolean);
        _fields.push(React.createElement(
          'span',
          { className: 'ra-doc-layout-itemsGroup', key: 'group' + key },
          fieldsToAdd
        ));
      } else {
        var fieldToAdd = getField(item);
        if (fieldToAdd) {
          _fields.push(fieldToAdd);
        }
      }
    };
    dashboardConfig.docLayout.map(function (item, index) {
      return addItemField(item, index, fields);
    });
  }
  return fields;
};
export default getDocFields;
